unit DblPointUnit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Math;

(*
    ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        TDblPoint
    ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*)
type
    TDblPoint = record
        X: double;
        Y: double;
    end;

function LimitRand(pMin, pMax: double): double;overload;
function LimitRand(pMin, pMax: integer): integer;overload;
function DblPoint(pX, pY: double): TDblPoint;
function MidPoint(aPoint, bPoint: TDblPoint): TDblPoint;
function Centroid(aPoint, bPoint, cPoint: TDblPoint): TDblPoint;
function CastPoint(aPoint: TDblPoint): TPoint;
function DistanceAtoBxx2(aPoint, bPoint: TDblPoint): double;
function DistanceAtoB(aPoint, bPoint: TDblPoint): double;
function RotatePoint(aCenter, aMove: TDblPoint; Radian: double): TDblPoint;
function GetEndPoint(OtherEnd: TDblPoint; Radius, Radian: double): TDblPoint;
procedure PlotDblPoint(aPoint: TDblPoint; aCanvas: TCanvas);
procedure ConnectDblPoint(aStart, aEnd: TDblPoint; aCanvas: TCanvas);overload;
procedure ConnectDblPoint(aPoints: array of TDblPoint; aCanvas: TCanvas);overload;
procedure DrawCircle(aCenter: TDblPoint; aRadius: double; fCanvas: TCanvas);
procedure DrawRadial(aCenter: TDblPoint; aPoints: array of TDblPoint; fCanvas: TCanvas);
procedure TranselatePoint(var aPoints: array of TDblPoint; fX, fY: Double);

const
    PI = System.Pi;

implementation

(*
    //////////////////////////////////////////////////////////
        Misc
    //////////////////////////////////////////////////////////
*)

//Eݒ\
function LimitRand(pMin, pMax: double): double;overload;
begin
    Result := Random * (pMax - pMin) + pMin;
end;
function LimitRand(pMin, pMax: integer): integer;overload;
begin
    Result := Random(pMax - pMin) + pMin;
end;


//TDblPoint֘A
//WTDblPint
function DblPoint(pX, pY: double): TDblPoint;
begin
    Result.X := pX;
    Result.Y := pY;
end;
//2_̒_
function MidPoint(aPoint, bPoint: TDblPoint): TDblPoint;
begin
    Result.X := (aPoint.X + bPoint.X) * 0.5;
    Result.Y := (aPoint.Y + bPoint.Y) * 0.5;
end;
//3_̏dS
function Centroid(aPoint, bPoint, cPoint: TDblPoint): TDblPoint;
begin
    Result.X := (aPoint.X + bPoint.X + cPoint.X) / 3;
    Result.Y := (aPoint.Y + bPoint.Y + cPoint.Y) / 3;
end;
//Point^ւ̃LXg
function CastPoint(aPoint: TDblPoint): TPoint;
begin
    Result := Point(Round(aPoint.X), Round(aPoint.Y));
end;

//2_Ԃ̋2
function DistanceAtoBxx2(aPoint, bPoint: TDblPoint): double;
var
    dx, dy: double;
begin
    dx := (aPoint.X - bPoint.X);
    dy := (aPoint.Y - bPoint.Y);
    Result := dx * dx + dy * dy;
end;
//2_Ԃ̋
function DistanceAtoB(aPoint, bPoint: TDblPoint): double;
begin
    Result := sqrt(DistanceAtoBxx2(aPoint, bPoint));
end;
//]̓_̍W
function RotatePoint(aCenter, aMove: TDblPoint; Radian: double): TDblPoint;
var
    aPoint: TDblPoint;
    sin, cos: Extended;
begin
    try
        aPoint := DblPoint(aMove.X - aCenter.X, aMove.Y - aCenter.Y);
        {
            x = rcos(a)
            y = rsin(a)
            -----------
            x' = rcos(a+b) = r(cos(a)cos(b) - sin(a)sin(b)) = xcos(b) - ysin(b)
            y' = rsin(a+b) = r(sin(a)cos(b) + cos(a)sin(b)) = ycos(b) + xsin(b)
        }
        SinCos(Radian, Sin, Cos);
        aPoint.X := (aPoint.X * cos - aPoint.Y * sin) + aCenter.X;
        aPoint.Y := (aPoint.Y * cos + aPoint.X * sin) + aCenter.Y;
        Result := aPoint;
    except
        ShowMessage('RotatePoint');
    end;
end;
//[_ƔaApx̒[_߂
function GetEndPoint(OtherEnd: TDblPoint; Radius, Radian: double): TDblPoint;
var
    sin, cos: Extended;
begin
    try
        SinCos(Radian, sin, cos);
        Result.X := Radius * cos + OtherEnd.X;
        Result.Y := Radius * sin + OtherEnd.Y;
    except
        ShowMessage('GetEndPoint');
    end;
end;
//w肳ꂽ_vbg
procedure PlotDblPoint(aPoint: TDblPoint; aCanvas: TCanvas);
var
    a_Point: TPoint;
begin
    try
        a_Point := CastPoint(aPoint);
//        aCanvas.Rectangle(a_Point.X - 5, a_Point.Y - 5, a_Point.X + 5, a_Point.Y + 5);
        aCanvas.Ellipse(a_Point.X - 5, a_Point.Y - 5, a_Point.X + 5, a_Point.Y + 5);
    except
        ShowMessage('PlotDblPoint');
    end;
end;
//w肳ꂽ_A
procedure ConnectDblPoint(aStart, aEnd: TDblPoint; aCanvas: TCanvas);overload;
var
    a_Point, b_Point: TPoint;
begin
    try
        a_Point := CastPoint(aStart);
        b_Point := CastPoint(aEnd);
        aCanvas.MoveTo(a_Point.X, a_Point.Y);
        aCanvas.LineTo(b_Point.X, b_Point.Y);
    except
        ShowMessage('ConnectDblPoint');
    end;
end;
//w肳ꂽ_A
procedure ConnectDblPoint(aPoints: array of TDblPoint; aCanvas: TCanvas);overload;
var
    i: integer;
    aLow, aHigh: integer;
begin
    try
        aLow  := Low(aPoints);
        aHigh := High(aPoints);
        if aLow = aHigh then Exit;
        for i := aLow to aHigh - 1 do begin
            ConnectDblPoint(aPoints[i], aPoints[i + 1], aCanvas);
        end;
    except
        ShowMessage('ConnectDblPoint');
    end;
end;
//w肳ꂽ_𒆐SƂĔaR̉~`
procedure DrawCircle(aCenter: TDblPoint; aRadius: double; fCanvas: TCanvas);
var
    aRect: TRect;
    aPoint: TPoint;
begin
    aPoint := CastPoint(DblPoint(aCenter.X - aRadius, aCenter.Y - aRadius));
    aRect.TopLeft := aPoint;
    aPoint := CastPoint(DblPoint(aCenter.X + aRadius, aCenter.Y + aRadius));
    aRect.BottomRight := aPoint;
    fCanvas.Ellipse(aRect);
end;
//SAe_ƕˏɌ
procedure DrawRadial(aCenter: TDblPoint; aPoints: array of TDblPoint; fCanvas: TCanvas);
var
    i: integer;
begin
    for i := Low(aPoints) to High(aPoints) do begin
        ConnectDblPoint(aCenter, aPoints[i], fCanvas);
    end;
end;
//_𕽍sړ
procedure TranselatePoint(var aPoints: array of TDblPoint; fX, fY: Double);
var
    i: integer;
begin
    for i := Low(aPoints) to High(aPoints) do begin
        aPoints[i].X := aPoints[i].X + fX;
        aPoints[i].Y := aPoints[i].X + fY;
    end;
end;

end.
